<?php

namespace Metropolitan\Tour\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Metropolitan\Tour\Forms\Tour as TourForm;
use Metropolitan\Tour\Http\Requests\TourResourceRequest;
use Metropolitan\Tour\Http\Resources\TourResource;
use Metropolitan\Tour\Http\Resources\ToursCollection;
use Metropolitan\Tour\Models\Tour;
use Metropolitan\Tour\Actions\TourAction;
use Metropolitan\Tour\Actions\TourActions;


/**
 * Resource controller class for tour.
 */
class TourResourceController extends BaseController
{

    /**
     * Initialize tour resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = TourForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('metropolitan.tour.modules'), 'tour', guard_url('tour'));
            return $next($request);
        });
    }

    /**
     * Display a list of tour.
     *
     * @return Response
     */
    public function index(TourResourceRequest $request)
    {
        $request = $request->all();
        $page = TourActions::run('paginate', $request);

        $data = new ToursCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('tour::tour.names'))
            ->view('tour::tour.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display tour.
     *
     * @param Request $request
     * @param Model   $tour
     *
     * @return Response
     */
    public function show(TourResourceRequest $request, Tour $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TourResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('tour::tour.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('tour::tour.show')
            ->output();
    }

    /**
     * Show the form for creating a new tour.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TourResourceRequest $request, Tour $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TourResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('tour::tour.name'))
            ->view('tour::tour.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new tour.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TourResourceRequest $request, Tour $model)
    {
        try {
            $request = $request->all();
            $model = TourAction::run('store', $model, $request);
            $data = new TourResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('tour::tour.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('tour/tour/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/tour/tour'))
                ->redirect();
        }

    }

    /**
     * Show tour for editing.
     *
     * @param Request $request
     * @param Model   $tour
     *
     * @return Response
     */
    public function edit(TourResourceRequest $request, Tour $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TourResource($model);
        // return view('tour::tour.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('tour::tour.name'))
            ->view('tour::tour.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the tour.
     *
     * @param Request $request
     * @param Model   $tour
     *
     * @return Response
     */
    public function update(TourResourceRequest $request, Tour $model)
    {
        try {
            $request = $request->all();
            $model = TourAction::run('update', $model, $request);
            $data = new TourResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('tour::tour.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('tour/tour/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('tour/tour/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the tour.
     *
     * @param Model   $tour
     *
     * @return Response
     */
    public function destroy(TourResourceRequest $request, Tour $model)
    {
        try {

            $request = $request->all();
            $model = TourAction::run('destroy', $model, $request);
            $data = new TourResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('tour::tour.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('tour/tour/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('tour/tour/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
